unit WSStripperComponent;

interface

uses
  Windows, Messages, SysUtils, Classes, WSStripperParser;

type
  TWSStripperParserComponent = class(TComponent)
  private
    { Private declarations }
    FWSStripperParser: TWSStripperParser;
    FInput: string;
    FOutput: string;
    procedure SetInput(const Value: string);
    procedure SetOutput(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property Input: string read FInput write SetInput;
    property Output: string read FOutput write SetOutput;
  end;


function ExecuteWSStripperParser(const sInput: string; var sOutput: string): boolean;


implementation

uses WSStripperScanner;


function ExecuteWSStripperParser(const sInput: string; var sOutput: string): boolean;
begin
  with  TWSStripperParserComponent.Create(nil) do
    try
      Input := sInput;
      Result := Execute;
      sOutput := Output;
    finally
      Free;
    end;
end;


{ TWSStripperParserComponent }

constructor TWSStripperParserComponent.Create(Aowner: TComponent);
begin
  inherited;
  FWSStripperParser := TWSStripperParser.Create;
end;

destructor TWSStripperParserComponent.Destroy;
begin
  FWSStripperParser.Free;
  inherited;
end;

function TWSStripperParserComponent.Execute: boolean;
begin
  FWSStripperParser.SourceString := FInput;
  FWSStripperParser.Parse;
  FOutput := FWSStripperParser.lst.DataString;
  Result := FWSStripperParser.errors = 0;
end;

procedure TWSStripperParserComponent.SetOutput(const Value: string);
begin
  FOutput := Value;
end;

procedure TWSStripperParserComponent.SetInput(const Value: string);
begin
  FInput := Value;
end;

end.
