// Generated by ParserBuilder @ 19/05/2001 4:25:28 PM

unit WSStripperParser;

(* NOTE: Any changes made to this file may be overwritten when it gets regenerated
  from within ParserBuilder - unless you run the Delphi Import command*)


interface

uses
  Windows, Messages, SysUtils, Classes, WSStripperScanner;


const
  maxT = 4;
  minErrDist = 2;  (* minimal distance (good tokens) between two errors *)
  setsize = 16;  (* sets are stored in 16 bits *)

type
  BITSET = set of 0..15;
  TSymbolSet = array [0..maxT div setsize] of BITSET;

  {PBGLOBAL}

  {ENDPBGLOBAL}




type
  TWSStripperParser = class(TWSStripperScanner)
  private
    FSuccessful: boolean;
    {PBPRIVATE}
    fOutputStream: TStringStream;
    fCollectStart: Integer;
    fCollectLength: Integer;
    procedure Init;
    procedure Final;
    procedure WriteStr(S: string);
    function GetLexString: string;
    procedure ReportError(nr, line, col, pos: Integer);

    {ENDPBPRIVATE}
    procedure PerformGet;
    procedure SynError(errNo: Integer);
    function _In(var s: TSymbolSet; x: Integer): boolean;
    procedure Expect(n: Integer);
    function WeakSeparator(n, syFol, repFol: Integer): boolean;
    function GetSuccessful: boolean;
    procedure SetSourceString(const Value: string);
    procedure _WSStripper;


  protected
    {PBPROTECTED}
     
    {ENDPBPROTECTED}
  public
    {PBPUBLIC}
    
    {ENDPBPUBLIC}
    constructor Create;
    destructor Destroy; override;
    procedure Parse;
    procedure LexString(var Lex: string);
    procedure LexName(var Lex: string);
    procedure LookAheadString(var Lex: string);
    procedure LookAheadName(var Lex: string);
  published
    property Successful: boolean read GetSuccessful;
    property SourceString: string write SetSourceString;
    {PBPUBLISHED}
    
    {ENDPBPUBLISHED}
  end;

implementation

var
  symSet: array [0..0] of TSymbolSet;
  errDist: integer;
  sym: integer;

{  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
   + For ParserBuilder's Delphi import function to work,        -
   + you must place your code in this Arbitrary Source section. -
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+- }

  {PBARBITRARYSOURCE}


procedure TWSStripperParser.Init;
begin
  fOutputStream.Size := 0;
  fCollectStart := 0;
  fCollectLength := 0;
  lst.Size := 0;
end;

procedure TWSStripperParser.Final;
begin
  if Errors > 0 then
    lst.WriteString('*** Errors ***');
  lst.WriteString(fOutputStream.DataString);
  fOutputStream.Size := 0;
end;

procedure TWSStripperParser.WriteStr(S: string);
begin
  fOutputStream.WriteBuffer(S[1], length(S));
end;

function TWSStripperParser.GetLexString: string;
begin
  LexString(Result);
end;

procedure TWSStripperParser.ReportError(nr, line, col, pos: Integer);
var
  S, S1: string;
  SL: TStringList;
begin
  case nr of
    {$I  WSStripper.err}
  end; // Case
  S := CRLF + S + CRLF + 'Line ' + IntToStr(line) + CRLF + 'Column ' +
    IntToStr(col) + CRLF + '................................' + CRLF;
  SL := TStringList.Create;
  SL.Text := src.DataString;
  if Line < SL.Count then
    S1 := SL[Line - 1]
  else 
    S1 := '[End Of Source File]';
  SL.Free;
  Insert(' ERROR->', S1, Col - 1);
  Inc(fErrors);
  WriteStr(S + S1 + CRLF);
end;




{ENDPBARBITRARYSOURCE}

 { TWSStripperParser - Generated Methods below}

procedure TWSStripperParser.SynError(errNo: Integer);
begin
  if errDist >= minErrDist then
  begin
    Error(errNo, nextLine, nextCol, nextPos);
  end;
  errDist := 0;
end;

procedure TWSStripperParser.PerformGet;
begin
  repeat
    Get(sym);
    if sym <= maxT then
      INC(errDist);
  until sym <= maxT
  end;

function TWSStripperParser._In(var s: TSymbolSet; x: Integer): boolean;
begin
  _In := x mod setsize in s[x div setsize];
end;

function TWSStripperParser.GetSuccessful: boolean;
begin
  FSuccessful := errors = 0;
  Result := FSuccessful;
end;

procedure TWSStripperParser.Expect(n: Integer);
begin
  if Sym = n then PerformGet
  else
    SynError(n);
end;


function TWSStripperParser.WeakSeparator(n, syFol, repFol: Integer): boolean;
var
  s: TSymbolSet;
  i: integer;
begin
  if sym = n then
  begin
    PerformGet;
    WeakSeparator := True;
    EXIT;
  end
  else if _In(symSet[repFol], sym) then
  begin
    WeakSeparator := False;
    exit
  end
  else
  begin
    i := 0;
    while i <= maxT div setsize do
    begin
      s[i] := symSet[0, i] + symSet[syFol, i] + symSet[repFol, i];
      INC(i)
    end;
    SynError(n);
    while not _In(s, sym) do PerformGet;
    WeakSeparator := _In(symSet[syFol], sym)
  end
end;

procedure TWSStripperParser.LexName(var Lex: string);
begin
  GetName(pos, len, Lex)
end;

procedure TWSStripperParser.LexString(var Lex: string);
begin
  GetString(pos, len, Lex)
end;

procedure TWSStripperParser.LookAheadName(var Lex: string);
begin
  GetName(nextPos, nextLen, Lex);
end;

procedure TWSStripperParser.LookAheadString(var Lex: string);
begin
  GetString(nextPos, nextLen, Lex)
end;

procedure TWSStripperParser.SetSourceString(const Value: string);
begin
  if (src <> nil) then
    src.Free;
  src := TStringStream.Create(Value);
end;

procedure TWSStripperParser._WSStripper;
begin
  Init;
  while (sym < 16) (* prevent range error *) and
    (sym in [1, 2, 3, 4]) do 
  begin
    PerformGet;
    WriteStr(GetLexString);
  end;
  Expect(0);
  Final;
end;



procedure TWSStripperParser.Parse;
begin
  if (src.DataString = '') then 
    lst.WriteString('Empty Source')
  else 
  begin    
    _Reset; 
    PerformGet;
    _WSStripper;
  end;
end;

destructor TWSStripperParser.Destroy;
begin
  {PBDESTROY}
  fOutputStream.Free;

  {ENDPBDESTROY}
  inherited;
end;

constructor TWSStripperParser.Create;
begin
  inherited;
  {PBCREATE}
  fOutputStream := TStringStream.Create('');
  Error := ReportError;

  {ENDPBCREATE}
  errDist := minErrDist;
  symSet[0, 0] := [0];
end;


end.
