{-----------------------------------------------------------------------------
 Unit Name: NewATGWizard
 Author:     pat connors
 Purpose: Component that manages the process of building a new wizard.
-----------------------------------------------------------------------------}


unit NewATGWizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, NewATGWizard_Main, Controls, pbStrLib,
  FileUtil;

type
  TNewATGWizard = class(TComponent)
  private
    { Private declarations }
    fWizForm: TfrmNewATGWizard;
    FATG: string;
    FFileName: string;
    FCaseSensitive: boolean;
    FIgnoreCtrlChars: boolean;
    FNestedComments: boolean;
    FHasComments: boolean;
    FNumRecognitionType: Integer;
    FCommentStart: string;
    FCommentEnd: string;
    fFrameDir: string;
    procedure SetATG(const Value: string);
    procedure SetFileName(const Value: string);
    procedure SetCaseSensitive(const Value: boolean);
    procedure SetCommentEnd(const Value: string);
    procedure SetCommentStart(const Value: string);
    procedure SetHasComments(const Value: boolean);
    procedure SetIgnoreCtrlChars(const Value: boolean);
    procedure SetNestedComments(const Value: boolean);
    procedure SetNumRecognitionType(const Value: Integer);
    function RunWizard: boolean;
    procedure BuildATG;
    function GetATGHeading: string;
    function GetDelphiSection: string;
    function GetCharacters: string;
    function GetTokens: string;
    function GetIgnore: string;
    function GetComments: string;
    function GetProductions: string;

  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property FileName: string read FFileName write SetFileName;
    property ATG: string read FATG write SetATG;
    property NumRecognitionType: Integer read FNumRecognitionType write SetNumRecognitionType;
    property CaseSensitive: boolean read FCaseSensitive write SetCaseSensitive;
    property IgnoreCtrlChars: boolean read FIgnoreCtrlChars write SetIgnoreCtrlChars;
    property HasComments: boolean read FHasComments write SetHasComments;
    property CommentStart: string read FCommentStart write SetCommentStart;
    property CommentEnd: string read FCommentEnd write SetCommentEnd;
    property NestedComments: boolean read FNestedComments write SetNestedComments;
  end;

implementation

uses StdCtrls;

{ TNewATGWizard }

procedure TNewATGWizard.BuildATG;
var 
  Dir: string;
begin
  // Do the actual construction of the ATG file.
  // - the WizForm is still valid, - it will be mined for
  // data to be stored in local properties.
  with fWizForm do
  begin
    // set project directory

    Dir := RemoveBackSlash(DirectoryEdit1.LongName) + '\';

    if CheckBox3.Checked then
      Dir := Dir + edParserName.Text + '\';

    // If Directory doesn't exist, then create it
    if not DirectoryExists(Dir) then
      ForceDirectories(Dir);

    FFileName := Dir + edParserName.Text + '.atg';
    FCaseSensitive := JvRadioGroup2.ItemIndex = 1;
    FIgnoreCtrlChars := JvRadioGroup3.ItemIndex = 1;
    FNestedComments := Checkbox2.Checked;
    FHasComments := CheckBox1.Checked;
    FNumRecognitionType := JvRadioGroup1.ItemIndex;
    FCommentStart := Edit1.Text;
    FCommentEnd := Edit2.Text;
  end;
  FATG := GetATGHeading;
  FATG := FATG + GetDelphiSection;
  if (not FCaseSensitive) then
    FATG := FATG + #13#10 + 'IGNORE CASE' + #13#10;
  FATG := FATG + CRLF + GetCharacters;
  FATG := FATG + CRLF + GetTokens;
  FATG := FATG + CRLF + GetIgnore;
  FATG := FATG + CRLF + GetComments;
  FATG := FATG + CRLF + GetProductions;
end;

constructor TNewATGWizard.Create(AOwner: TComponent);
begin
  inherited;
  fWizForm := nil;
  fATG := '';
  fFileName := '';
end;

destructor TNewATGWizard.Destroy;
begin
  inherited;
end;

function TNewATGWizard.Execute: boolean;
begin
  fFrameDir := RemoveBackSlash(ExtractFilePath(Application.ExeName)) + '\Frames\';
  Result := RunWizard;
end;

function TNewATGWizard.GetATGHeading: string;
begin
  Result := '$C COMPILER ' + Trim(fWizForm.edParserName.Text) + #13#10;
end;

function TNewATGWizard.GetCharacters: string;
begin
  if (FileExists(fFrameDir + 'NewATG_Chars.txt')) then
    LoadString(Result, fFrameDir + 'NewATG_Chars.txt')
  else
    Result := fWizForm.CharStrings.Strings.Text;
end;

function TNewATGWizard.GetComments: string;
begin
  if (FHasComments) then
  begin
    Result := #13#10 + 'COMMENTS FROM "' + FCommentStart + '" TO "' + FCommentEnd + '" ';
    if (FNestedComments) then
      Result := Result + 'NESTED';
  end
  else
    Result := '';
end;

function TNewATGWizard.GetDelphiSection: string;
begin
  if (FileExists(fFrameDir + 'NewATG_Delphi.txt')) then
    LoadString(Result, fFrameDir + 'NewATG_Delphi.txt')
  else
    Result := fWizForm.DelphiStrings.Strings.Text;
end;

function TNewATGWizard.GetIgnore: string;
begin
  if (FIgnoreCtrlChars) then
  begin
    if (FileExists(fFrameDir + 'NewATG_Ignore.txt')) then
      LoadString(Result, fFrameDir + 'NewATG_Ignore.txt')
    else
      Result := #13#10 + 'IGNORE CHR(1)..CHR(31)';
  end
  else
    Result := '';
end;

function TNewATGWizard.GetProductions: string;
begin
  if (FileExists(fFrameDir + 'NewATG_Productions.txt')) then
    LoadString(Result, fFrameDir + 'NewATG_Productions.txt')
  else
    Result := #13#10 + 'PRODUCTIONS' + #13#10 + Trim(fWizForm.edParserName.Text) +
      '= (. Init; .) { ANY } EOF (. Final; .) .' + #13#10 + 'END ' +
      Trim(fWizForm.edParserName.Text) + '.';

  Result := ReplaceAll(Result, '-->Grammar', Trim(fWizForm.edParserName.Text));
end;

function TNewATGWizard.GetTokens: string;
begin
  case fWizForm.JvRadioGroup1.ItemIndex of
    0: if (FileExists(fFrameDir + 'NewATG_Tokens.txt')) then
        LoadString(Result, fFrameDir + 'NewATG_Tokens.txt')
      else 
        Result := 'TOKENS' + #13#10 + '  ident = (letter | digit) {letter | digit}.' +
          #13#10 +
          'string = ''"''{noquote1}(''"''|eol)|"''"{noquote2}("''"|eol).';
    1: if (FileExists(fFrameDir + 'NewATG_TokensInteger.txt')) then
        LoadString(Result, fFrameDir + 'NewATG_TokensInteger.txt')
      else  
        Result := fWizForm.tokenStrings.Strings.Text;
    2: if (FileExists(fFrameDir + 'NewATG_TokensFloat.txt')) then
        LoadString(Result, fFrameDir + 'NewATG_TokensFloat.txt')
      else 
        fWizForm.tokens2.Strings.Text;
  end;
end;

function TNewATGWizard.RunWizard: boolean;
var
  mr: TModalResult;
begin
  Result := False;
  fWizForm := TfrmNewATGWizard.Create(Self);
  try
    fWizForm.DirectoryEdit1.Text := ExtractFilePath(Application.ExeName) + 'Output';
    mr := fWizForm.ShowModal;
    if (mr = mrOk) then
    begin
      BuildATG;
      Result := True;
    end;
  finally
    fWizForm.Free;
    fWizForm := nil;
  end;
end;

procedure TNewATGWizard.SetATG(const Value: string);
begin
  FATG := Value;
end;

procedure TNewATGWizard.SetCaseSensitive(const Value: boolean);
begin
  FCaseSensitive := Value;
end;

procedure TNewATGWizard.SetCommentEnd(const Value: string);
begin
  FCommentEnd := Value;
end;

procedure TNewATGWizard.SetCommentStart(const Value: string);
begin
  FCommentStart := Value;
end;

procedure TNewATGWizard.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TNewATGWizard.SetHasComments(const Value: boolean);
begin
  FHasComments := Value;
end;

procedure TNewATGWizard.SetIgnoreCtrlChars(const Value: boolean);
begin
  FIgnoreCtrlChars := Value;
end;

procedure TNewATGWizard.SetNestedComments(const Value: boolean);
begin
  FNestedComments := Value;
end;

procedure TNewATGWizard.SetNumRecognitionType(const Value: Integer);
begin
  FNumRecognitionType := Value;
end;

end.
