unit SemActionFormatterComponent;

interface

uses
  Windows, Messages, SysUtils, Classes, SemActionFormatterParser, pbStrLib;

type
  TSemActionFormatterParserComponent = class(TComponent)
  private
    { Private declarations }
    FSemActionFormatterParser: TSemActionFormatterParser;
    FInput: string;
    FOutput: string;
    fStartIndent: Integer;
    fMinIndent: Integer;
    procedure SetInput(const Value: string);
    procedure SetOutput(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property Input: string read FInput write SetInput;
    property Output: string read FOutput write SetOutput;
    property MinIndent: Integer read fMinIndent write fMinIndent;
    property StartIndent: Integer read fStartIndent write fStartIndent;
  end;


function ExecuteSemActionFormatterParser(const sInput: string;
  const iMinIndent, iStartIndent: Integer; var sOutput: string): boolean;


implementation

uses SemActionFormatterScanner;


function ExecuteSemActionFormatterParser(const sInput: string;
  const iMinIndent, iStartIndent: Integer; var sOutput: string): boolean;
begin
  with  TSemActionFormatterParserComponent.Create(nil) do
    try
      Input := sInput;
      MinIndent := iMinIndent;
      StartIndent := iStartIndent;
      Result := Execute;
      sOutput := Output;
    finally
      Free;
    end;
end;


{ TSemActionFormatterParserComponent }

constructor TSemActionFormatterParserComponent.Create(Aowner: TComponent);
begin
  inherited;
  FSemActionFormatterParser := TSemActionFormatterParser.Create;
end;

destructor TSemActionFormatterParserComponent.Destroy;
begin
  FSemActionFormatterParser.Free;
  inherited;
end;

function TSemActionFormatterParserComponent.Execute: boolean;
begin
  FSemActionFormatterParser.SourceString := FInput;
  FSemActionFormatterParser.MinIndent := fMinIndent;
  FSemActionFormatterParser.StartIndent := fStartIndent;
  FSemActionFormatterParser.Parse;
  FOutput := FSemActionFormatterParser.lst.DataString;
  Result := FSemActionFormatterParser.errors = 0;
end;

procedure TSemActionFormatterParserComponent.SetOutput(const Value: string);
begin
  FOutput := Value;
end;

procedure TSemActionFormatterParserComponent.SetInput(const Value: string);
begin
  FInput := Value;
end;

end.
