unit ProductionChecker;

interface

uses
  Windows, Messages, SysUtils, Classes, SimpleSyntaxCheckerParser, pbStrLib;

type
  TProductionChecker = class(TComponent)
  private
    { Private declarations }
    FSimpleSyntaxCheckerParser: TSimpleSyntaxCheckerParser;
    FText: string;
    FReport: string;
    procedure SetText(const Value: string);
    procedure SetReport(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property Text: string read FText write SetText;
    property Report: string read FReport write SetReport;
  end;


function IsProductionOK(const ProdText: string; var sReport: string): boolean;


implementation

uses SimpleSyntaxCheckerScanner;

function IsProductionOK(const ProdText: string; var sReport: string): boolean;
begin
  with  TProductionChecker.Create(nil) do
    try
      Text := ProdText;
      Result := Execute;
      sReport := ReplaceAll(Report, '<', '(. ');
      sReport := ReplaceAll(sReport, '>', ' .)');
    finally
      Free;
    end;
end;

{ TProductionChecker }

constructor TProductionChecker.Create(Aowner: TComponent);
begin
  inherited;
  FSimpleSyntaxCheckerParser := TSimpleSyntaxCheckerParser.Create;
end;

destructor TProductionChecker.Destroy;
begin
  FSimpleSyntaxCheckerParser.Free;
  inherited;
end;

function TProductionChecker.Execute: boolean;
begin
  FSimpleSyntaxCheckerParser.SourceString := FText;
  FSimpleSyntaxCheckerParser.Parse;
  FReport := FSimpleSyntaxCheckerParser.lst.DataString;
  Result := FSimpleSyntaxCheckerParser.errors = 0;
end;

procedure TProductionChecker.SetReport(const Value: string);
begin
  FReport := Value;
end;

procedure TProductionChecker.SetText(const Value: string);
begin
  FText := Value;
end;

end.
